import { Observable } from 'rxjs';
import * as constants from './constants';
import { file, fs, listr, log, semver, value } from './libs';
import * as npm from './util.npm';
import { orderByDepth, toPackages } from './util.package';
export async function loadSettings(options = {}) {
    const { spinner = false, npm = false } = options;
    if (spinner) {
        let result;
        const title = npm ? 'Loading module info locally and from NPM' : 'Loading module info locally';
        const task = {
            title,
            task: () => new Observable((observer) => {
                observer.next('Calculating number of modules...');
                (async () => {
                    const onReadUpdate = (e) => {
                        const { total, completed, pkg } = e;
                        const percent = value.round((completed / total) * 100);
                        let msg = `Querying ${total} modules (${percent}%)`;
                        msg = pkg ? `${msg}: ${pkg.name}` : msg;
                        observer.next(msg);
                    };
                    result = await read(Object.assign(Object.assign({}, options), { onReadUpdate }));
                    observer.complete();
                })();
            }),
        };
        await listr([task]).run();
        log.info();
        return result;
    }
    else {
        return read(options);
    }
}
async function read(options = {}) {
    const { onReadUpdate } = options;
    const path = await file.findClosestAncestor(process.cwd(), constants.CONFIG_FILE_NAME);
    if (!path) {
        return;
    }
    const yaml = await loadYaml(path);
    if (!yaml) {
        return;
    }
    const dir = fs.dirname(path);
    yaml.modules = yaml.modules.map((path) => fs.resolve(dir, path));
    let modules = await toPackages(yaml.modules);
    modules = orderByDepth(modules);
    const ignore = {
        paths: await ignorePaths(yaml, dir),
        names: yaml.ignore.names,
    };
    modules.forEach((pkg) => (pkg.isIgnored = isIgnored(pkg, ignore)));
    let completed = -1;
    const fireReadUpdate = (pkg) => {
        completed++;
        if (onReadUpdate) {
            const total = modules.length;
            onReadUpdate({ total, completed, pkg });
        }
    };
    fireReadUpdate();
    if (options.npm) {
        const list = modules.filter((pkg) => !pkg.isIgnored);
        const npmModules = await npm.info(list, { onModuleRead: fireReadUpdate });
        modules.forEach((pkg) => {
            var _a;
            pkg.npm = npmModules.find((item) => item.name === pkg.name);
            if (((_a = pkg.npm) === null || _a === void 0 ? void 0 : _a.latest) && semver.gt(pkg.npm.latest, pkg.version)) {
                pkg.latest = pkg.npm.latest;
            }
        });
    }
    return {
        path,
        ignored: ignore,
        watchPattern: yaml.watchPattern,
        modules,
    };
}
async function ignorePaths(yaml, dir) {
    const result = [];
    for (const pattern of yaml.ignore.paths) {
        const paths = await fs.glob.find(fs.resolve(dir, pattern));
        paths.forEach((path) => result.push(path));
    }
    return result;
}
function isIgnored(pkg, ignore) {
    if (ignore.names.includes(pkg.name)) {
        return true;
    }
    for (const path of ignore.paths) {
        if (pkg.dir.startsWith(path)) {
            return true;
        }
    }
    return false;
}
async function loadYaml(path) {
    try {
        const result = await file.yaml(path);
        result.modules = result.modules || [];
        result.ignore = result.ignore || { paths: [] };
        result.ignore.paths = result.ignore.paths || [];
        result.ignore.names = result.ignore.names || [];
        result.watchPattern = result.watchPattern || constants.DEFAULT_WATCH_PATTERN;
        return result;
    }
    catch (error) {
        log.error(`Failed to parse YAML configuration.`);
        log.error(error.message);
        log.info(log.magenta('File:'), path, '\n');
    }
    return;
}
